using System;
using System.Xml;
using Microsoft.Web.Services2;
using Microsoft.Web.Services2.Addressing;
using Microsoft.Web.Services2.Messaging;
using Microsoft.Web.Services2.Security;
using Microsoft.Web.Services2.Security.Tokens;
using Microsoft.Web.Services2.Security.X509;

public class Loan
{
	public string name;
	public int amount;
}

class LoanServiceClient
{
    static void Main(string[] args)
    {
        try
        {
			SoapEnvelope message = new SoapEnvelope();

			Uri destination   = new Uri("soap.tcp://localhost/LoanService");
			EndpointReference er = new EndpointReference(destination);
	
			Uri reply = new Uri("soap.tcp://localhost:8082/LoanRequest");
			ReplyTo replyER = new ReplyTo(reply);
            message.Context.Addressing.ReplyTo = replyER;

			FaultTo faultER = new FaultTo(reply);
            message.Context.Addressing.FaultTo = faultER;

			SoapReceivers.Add(replyER, typeof(LoanResponse));

			message.Context.Addressing.Action = new Action( "urn:LoanRequest");

			X509Certificate clientCert = Certificate.GetCertificate(Certificate.clientCert);
			X509SecurityToken clientToken = new X509SecurityToken(clientCert);
			message.Context.Security.Tokens.Add(clientToken);

			MessageSignature signedData = new MessageSignature(clientToken);
			message.Context.Security.Elements.Add(signedData);
			
			Loan loan = new Loan();
			loan.name = "Mary Smith";
			loan.amount = 1000;
			message.SetBodyObject(loan);

			SoapSender sender = new SoapSender(er);
			sender.Send(message);

			Console.WriteLine("Loan Request sent to {0}", sender.Destination.Address.Value);
		}
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
        }

        Console.WriteLine( "" );
        Console.WriteLine("Press [Enter] to continue...");
        Console.WriteLine( "" );
        Console.ReadLine();
    }
}

public class LoanResponse : SoapReceiver
{
    protected override void Receive(SoapEnvelope message)
    {
		if (message.Fault != null)
			Console.WriteLine(message.Fault.ToString());
		else
		{
			string result = (string)message.GetBodyObject(typeof(string));
			Console.WriteLine("Loan Result: " + result);
		}
    }
}



















