using System;
using System.Web.Services.Protocols;

using Microsoft.Web.Services2;
using Microsoft.Web.Services2.Addressing;
using Microsoft.Web.Services2.Messaging;
using Microsoft.Web.Services2.Security;
using Microsoft.Web.Services2.Security.Tokens;
using Microsoft.Web.Services2.Security.X509;

public class Loan
{
	public string name;
	public int amount;
}

public class LoanService : SoapReceiver
{
    protected override void Receive( SoapEnvelope message )
    {
        if( message.Context.Addressing.Action.Value == "urn:LoanRequest" )
        {
			Loan loan = (Loan)message.GetBodyObject(typeof(Loan));
            Console.WriteLine("Loan Request received for {0} in the amount of {1}", loan.name, loan.amount);

			EndpointReference er = new EndpointReference(message.Context.Addressing.ReplyTo.Address.Value);
	
			SoapEnvelope response = new SoapEnvelope();
			response.Context.Addressing.Action = new Action("urn:LoanResponse");

			string result = "not approved";
			bool valid = LoanService.ProcessClientCredentials(message.Context);
			if (valid == true)
				result = "approved.";

			bool foundSignedData = false;
			foreach (ISecurityElement se in message.Context.Security.Elements)
			{
				if (se is MessageSignature)
				{
					foundSignedData = true;
					break;
				}
			}

			if (foundSignedData == false)
				throw new SoapException("Message should be signed", SoapException.ClientFaultCode);     

			response.SetBodyObject(result);
			
			SoapSender sender = new SoapSender(er);
			sender.Send(response);
       }
    }

	static bool ProcessClientCredentials(SoapContext context)
	{
		SecurityTokenCollection requestTokens = context.Security.Tokens;

		bool valid = false;
		X509Certificate cert = null;

		foreach (SecurityToken token in requestTokens)
		{
			X509SecurityToken tok = token as X509SecurityToken;
			if (tok != null)
			{				
				cert = tok.Certificate;
				valid = Certificate.IsClientCertificateValid(cert);
				if (valid == true)
				{
					context["LoanOfficer"] = cert.GetName();
					break;
				}
			}
		}

		return valid;
	}

}














